@@ -83,7 +83,7 @@ def load_configuration(cfgfiles=None):
     CONFIG.set('server', 'temp_path', tempfile.gettempdir())
     CONFIG.set('server', 'processes_path', '')
     outputpath = tempfile.gettempdir()
-    CONFIG.set('server', 'outputurl', 'file://%s' % outputpath)
+    CONFIG.set('server', 'outputurl', 'file://{}'.format(outputpath))
     CONFIG.set('server', 'outputpath', outputpath)
     # list of allowed input paths (file url input) seperated by ':'
     CONFIG.set('server', 'allowedinputpaths', '')
@@ -95,6 +95,7 @@ def load_configuration(cfgfiles=None):
     # If this flag is enabled PyWPS will remove the process temporary workdir
     # after process has finished.
     CONFIG.set('server', 'cleantempdir', 'true')
+    CONFIG.set('server', 'storagetype', 'file')
 
     CONFIG.add_section('processing')
     CONFIG.set('processing', 'mode', 'default')
@@ -115,7 +116,7 @@ def load_configuration(cfgfiles=None):
     CONFIG.set('metadata:main', 'identification_fees', 'NONE')
     CONFIG.set('metadata:main', 'identification_accessconstraints', 'NONE')
     CONFIG.set('metadata:main', 'provider_name', 'Organization Name')
-    CONFIG.set('metadata:main', 'provider_url', 'http://pywps.org/')
+    CONFIG.set('metadata:main', 'provider_url', 'https://pywps.org/')
     CONFIG.set('metadata:main', 'contact_name', 'Lastname, Firstname')
     CONFIG.set('metadata:main', 'contact_position', 'Position Title')
     CONFIG.set('metadata:main', 'contact_address', 'Mailing Address')
@@ -134,6 +135,13 @@ def load_configuration(cfgfiles=None):
     CONFIG.add_section('grass')
     CONFIG.set('grass', 'gisbase', '')
 
+    CONFIG.add_section('s3')
+    CONFIG.set('s3', 'bucket', '')
+    CONFIG.set('s3', 'prefix', '')
+    CONFIG.set('s3', 'public', 'false')
+    CONFIG.set('s3', 'encrypt', 'false')
+    CONFIG.set('s3', 'region', '')
+
     if not cfgfiles:
         cfgfiles = _get_default_config_files_location()
 
@@ -142,7 +150,7 @@ def load_configuration(cfgfiles=None):
 
     loaded_files = CONFIG.read(cfgfiles)
     if loaded_files:
-        LOGGER.info('Configuration file(s) %s loaded', loaded_files)
+        LOGGER.info('Configuration file(s) {} loaded'.format(loaded_files))
     else:
         LOGGER.info('No configuration files loaded. Using default values')
 
@@ -159,12 +167,11 @@ def _check_config():
         confvalue = get_config_value('server', confid)
 
         if not os.path.isdir(confvalue):
-            LOGGER.warning('server->%s configuration value %s is not directory'
-                           % (confid, confvalue))
+            LOGGER.warning('server->{} configuration value {} is not directory'.format(confid, confvalue))
 
         if not os.path.isabs(confvalue):
-            LOGGER.warning('server->%s configuration value %s is not absolute path, making it absolute to %s' %
-                           (confid, confvalue, os.path.abspath(confvalue)))
+            LOGGER.warning('server->{} configuration value {} is not absolute path, making it absolute to {}'.format(
+                           confid, confvalue, os.path.abspath(confvalue)))
             CONFIG.set('server', confid, os.path.abspath(confvalue))
 
     [checkdir(n) for n in ['workdir', 'outputpath']]
@@ -237,5 +244,5 @@ def get_size_mb(mbsize):
         newsize /= 1024
     else:
         newsize *= 1
-    LOGGER.debug('Calculated real size of %s is %s', mbsize, newsize)
+    LOGGER.debug('Calculated real size of {} is {}'.format(mbsize, newsize))
     return newsize
