@@ -3,28 +3,34 @@
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
+from pywps.translations import lower_case_dict
 from pywps._compat import text_type, StringIO
 import os
+from io import open
 import shutil
 import requests
 import tempfile
 import logging
 import pywps.configuration as config
 from pywps.inout.literaltypes import (LITERAL_DATA_TYPES, convert,
-                                      make_allowedvalues, is_anyvalue)
-from pywps import get_ElementMakerForVersion, OGCUNIT, NAMESPACES
+                                      make_allowedvalues, is_anyvalue,
+                                      is_values_reference)
+from pywps import OGCUNIT
 from pywps.validator.mode import MODE
 from pywps.validator.base import emptyvalidator
 from pywps.validator import get_validator
-from pywps.validator.literalvalidator import (validate_anyvalue,
-                                              validate_allowed_values)
+from pywps.validator.literalvalidator import (validate_value,
+                                              validate_anyvalue,
+                                              validate_allowed_values,
+                                              validate_values_reference)
 from pywps.exceptions import NoApplicableCode, InvalidParameterValue, FileSizeExceeded, \
     FileURLNotSupported
 from pywps._compat import PY2, urlparse
 import base64
 from collections import namedtuple
+from copy import deepcopy
 from io import BytesIO
-import six
+
 
 _SOURCE_TYPE = namedtuple('SOURCE_TYPE', 'MEMORY, FILE, STREAM, DATA, URL')
 SOURCE_TYPE = _SOURCE_TYPE(0, 1, 2, 3, 4)
@@ -67,6 +73,9 @@ class UOM(object):
         return {"reference": OGCUNIT[self.uom],
                 "uom": self.uom}
 
+    def __eq__(self, other):
+        return self.uom == other.uom
+
 
 class IOHandler(object):
     """Base IO handling class subclassed by specialized versions: FileHandler, UrlHandler, DataHandler, etc.
@@ -234,6 +243,11 @@ class IOHandler(object):
         else:
             return ''
 
+    def clone(self):
+        """Create copy of yourself
+        """
+        return deepcopy(self)
+
     @staticmethod
     def _create_fset_properties():
         """Create properties that when set for the first time, will determine
@@ -285,7 +299,9 @@ class FileHandler(IOHandler):
     def data(self):
         """Read file and return content."""
         if self._data is None:
-            with open(self.file, mode=self._openmode()) as fh:
+            openmode = self._openmode()
+            kwargs = {} if 'b' in openmode else {'encoding': 'utf8'}
+            with open(self.file, mode=openmode, **kwargs) as fh:
                 self._data = fh.read()
         return self._data
 
@@ -366,7 +382,9 @@ class DataHandler(FileHandler):
         """
         if self._file is None:
             self._file = self._build_file_name()
-            with open(self._file, self._openmode(self.data)) as fh:
+            openmode = self._openmode(self.data)
+            kwargs = {} if 'b' in openmode else {'encoding': 'utf8'}
+            with open(self._file, openmode, **kwargs) as fh:
                 fh.write(self.data)
 
         return self._file
@@ -432,14 +450,13 @@ class UrlHandler(FileHandler):
             reference_file = self._openurl(self.url, self.post_data)
             data_size = reference_file.headers.get('Content-Length', 0)
         except Exception as e:
-            raise NoApplicableCode('File reference error: %s' % e)
+            raise NoApplicableCode('File reference error: {}'.format(e))
 
-        FSEE = FileSizeExceeded(
-            'File size for input {} exceeded. Maximum allowed: {} megabytes'.
-            format(getattr(self.inpt, 'identifier', '?'), max_byte_size))
+        error_message = 'File size for input "{}" exceeded. Maximum allowed: {} megabytes'.format(
+            self.inpt.get('identifier', '?'), max_byte_size)
 
         if int(data_size) > int(max_byte_size):
-            raise FSEE
+            raise FileSizeExceeded(error_message)
 
         try:
             with open(self._file, 'wb') as f:
@@ -447,7 +464,7 @@ class UrlHandler(FileHandler):
                 for chunk in reference_file.iter_content(chunk_size=1024):
                     data_size += len(chunk)
                     if int(data_size) > int(max_byte_size):
-                        raise FSEE
+                        raise FileSizeExceeded(error_message)
                     f.write(chunk)
 
         except Exception as e:
@@ -467,7 +484,7 @@ class UrlHandler(FileHandler):
     def _openurl(href, data=None):
         """Open given href.
         """
-        LOGGER.debug('Fetching URL %s', href)
+        LOGGER.debug('Fetching URL {}'.format(href))
         if data is not None:
             req = requests.post(url=href, data=data, stream=True)
         else:
@@ -527,8 +544,9 @@ class SimpleHandler(DataHandler):
 class BasicIO:
     """Basic Input/Output class
     """
+
     def __init__(self, identifier, title=None, abstract=None, keywords=None,
-                 min_occurs=1, max_occurs=1, metadata=[]):
+                 min_occurs=1, max_occurs=1, metadata=[], translations=None):
         self.identifier = identifier
         self.title = title
         self.abstract = abstract
@@ -536,6 +554,7 @@ class BasicIO:
         self.min_occurs = int(min_occurs)
         self.max_occurs = int(max_occurs)
         self.metadata = metadata
+        self.translations = lower_case_dict(translations)
 
 
 class BasicLiteral:
@@ -637,11 +656,10 @@ class BasicComplex(object):
             if not data_format.validate or data_format.validate == emptyvalidator:
                 data_format.validate = get_validator(data_format.mime_type)
         else:
-            raise InvalidParameterValue("Requested format "
-                                        "%s, %s, %s not supported" %
-                                        (data_format.mime_type,
-                                         data_format.encoding,
-                                         data_format.schema),
+            raise InvalidParameterValue("Requested format {}, {}, {} not supported".format(
+                                        data_format.mime_type,
+                                        data_format.encoding,
+                                        data_format.schema),
                                         'mimeType')
 
     def _is_supported(self, data_format):
@@ -662,8 +680,20 @@ class BasicBoundingBox(object):
         self.crss = crss or ['epsg:4326']
         self.crs = self.crss[0]
         self.dimensions = dimensions
-        self.ll = []
-        self.ur = []
+
+    @property
+    def ll(self):
+        data = getattr(self, 'data', None)
+        if data:
+            return data[:2]
+        return []
+
+    @property
+    def ur(self):
+        data = getattr(self, 'data', None)
+        if data:
+            return data[2:]
+        return []
 
 
 class LiteralInput(BasicIO, BasicLiteral, SimpleHandler):
@@ -674,18 +704,35 @@ class LiteralInput(BasicIO, BasicLiteral, SimpleHandler):
                  data_type="integer", workdir=None, allowed_values=None,
                  uoms=None, mode=MODE.NONE,
                  min_occurs=1, max_occurs=1, metadata=[],
-                 default=None, default_type=SOURCE_TYPE.DATA):
-        BasicIO.__init__(self, identifier, title, abstract, keywords,
-                         min_occurs, max_occurs, metadata)
+                 default=None, default_type=SOURCE_TYPE.DATA, translations=None):
+        BasicIO.__init__(self,
+                         identifier=identifier,
+                         title=title,
+                         abstract=abstract,
+                         keywords=keywords,
+                         min_occurs=min_occurs,
+                         max_occurs=max_occurs,
+                         metadata=metadata,
+                         translations=translations,
+                         )
         BasicLiteral.__init__(self, data_type, uoms)
         SimpleHandler.__init__(self, workdir, data_type, mode=mode)
 
         if default_type != SOURCE_TYPE.DATA:
             raise InvalidParameterValue("Source types other than data are not supported.")
 
-        self.any_value = is_anyvalue(allowed_values)
+        self.any_value = False
+        self.values_reference = None
         self.allowed_values = []
-        if not self.any_value:
+
+        if allowed_values:
+            if not isinstance(allowed_values, (tuple, list)):
+                allowed_values = [allowed_values]
+            self.any_value = any(is_anyvalue(a) for a in allowed_values)
+            for value in allowed_values:
+                if is_values_reference(value):
+                    self.values_reference = value
+                    break
             self.allowed_values = make_allowedvalues(allowed_values)
 
         self._default = default
@@ -702,33 +749,12 @@ class LiteralInput(BasicIO, BasicLiteral, SimpleHandler):
 
         if self.any_value:
             return validate_anyvalue
-        else:
+        elif self.values_reference:
+            return validate_values_reference
+        elif self.allowed_values:
             return validate_allowed_values
-
-    @property
-    def json(self):
-        """Get JSON representation of the input
-        """
-        data = {
-            'identifier': self.identifier,
-            'title': self.title,
-            'abstract': self.abstract,
-            'keywords': self.keywords,
-            'type': 'literal',
-            'data_type': self.data_type,
-            'workdir': self.workdir,
-            'any_value': self.any_value,
-            'allowed_values': [value.json for value in self.allowed_values],
-            'mode': self.valid_mode,
-            'data': self.data,
-            'min_occurs': self.min_occurs,
-            'max_occurs': self.max_occurs
-        }
-        if self.uoms:
-            data["uoms"] = [uom.json for uom in self.uoms],
-        if self.uom:
-            data["uom"] = self.uom.json
-        return data
+        else:
+            return validate_value
 
 
 class LiteralOutput(BasicIO, BasicLiteral, SimpleHandler):
@@ -737,8 +763,8 @@ class LiteralOutput(BasicIO, BasicLiteral, SimpleHandler):
 
     def __init__(self, identifier, title=None, abstract=None, keywords=None,
                  data_type=None, workdir=None, uoms=None, validate=None,
-                 mode=MODE.NONE):
-        BasicIO.__init__(self, identifier, title, abstract, keywords)
+                 mode=MODE.NONE, translations=None):
+        BasicIO.__init__(self, identifier, title, abstract, keywords, translations=translations)
         BasicLiteral.__init__(self, data_type, uoms)
         SimpleHandler.__init__(self, workdir=None, data_type=data_type,
                                mode=mode)
@@ -769,9 +795,17 @@ class BBoxInput(BasicIO, BasicBoundingBox, DataHandler):
                  dimensions=None, workdir=None,
                  mode=MODE.SIMPLE,
                  min_occurs=1, max_occurs=1, metadata=[],
-                 default=None, default_type=SOURCE_TYPE.DATA):
-        BasicIO.__init__(self, identifier, title, abstract, keywords,
-                         min_occurs, max_occurs, metadata)
+                 default=None, default_type=SOURCE_TYPE.DATA, translations=None):
+        BasicIO.__init__(self,
+                         identifier=identifier,
+                         title=title,
+                         abstract=abstract,
+                         keywords=keywords,
+                         min_occurs=min_occurs,
+                         max_occurs=max_occurs,
+                         metadata=metadata,
+                         translations=translations,
+                         )
         BasicBoundingBox.__init__(self, crss, dimensions)
         DataHandler.__init__(self, workdir=workdir, mode=mode)
 
@@ -783,47 +817,16 @@ class BBoxInput(BasicIO, BasicBoundingBox, DataHandler):
 
         self._set_default_value(default, default_type)
 
-    @property
-    def json(self):
-        """Get JSON representation of the input. It returns following keys in
-        the JSON object:
-
-            * identifier
-            * title
-            * abstract
-            * type
-            * crs
-            * bbox
-            * dimensions
-            * workdir
-            * mode
-        """
-        return {
-            'identifier': self.identifier,
-            'title': self.title,
-            'abstract': self.abstract,
-            'keywords': self.keywords,
-            'type': 'bbox',
-            'crs': self.crs,
-            'crss': self.crss,
-            'bbox': (self.ll, self.ur),
-            'dimensions': self.dimensions,
-            'workdir': self.workdir,
-            'mode': self.valid_mode,
-            'min_occurs': self.min_occurs,
-            'max_occurs': self.max_occurs
-        }
-
 
 class BBoxOutput(BasicIO, BasicBoundingBox, DataHandler):
     """Basic BoundingBox output class
     """
 
     def __init__(self, identifier, title=None, abstract=None, keywords=None, crss=None,
-                 dimensions=None, workdir=None, mode=MODE.NONE):
-        BasicIO.__init__(self, identifier, title, abstract, keywords)
+                 dimensions=None, workdir=None, mode=MODE.NONE, translations=None):
+        BasicIO.__init__(self, identifier, title, abstract, keywords, translations=translations)
         BasicBoundingBox.__init__(self, crss, dimensions)
-        DataHandler.__init__(self, workdir=None, mode=mode)
+        DataHandler.__init__(self, workdir=workdir, mode=mode)
         self._storage = None
 
     @property
@@ -848,10 +851,17 @@ class ComplexInput(BasicIO, BasicComplex, IOHandler):
                  workdir=None, data_format=None, supported_formats=None,
                  mode=MODE.NONE,
                  min_occurs=1, max_occurs=1, metadata=[],
-                 default=None, default_type=SOURCE_TYPE.DATA):
-
-        BasicIO.__init__(self, identifier, title, abstract, keywords,
-                         min_occurs, max_occurs, metadata)
+                 default=None, default_type=SOURCE_TYPE.DATA, translations=None):
+        BasicIO.__init__(self,
+                         identifier=identifier,
+                         title=title,
+                         abstract=abstract,
+                         keywords=keywords,
+                         min_occurs=min_occurs,
+                         max_occurs=max_occurs,
+                         metadata=metadata,
+                         translations=translations,
+                         )
         IOHandler.__init__(self, workdir=workdir, mode=mode)
         BasicComplex.__init__(self, data_format, supported_formats)
 
@@ -872,10 +882,10 @@ class ComplexInput(BasicIO, BasicComplex, IOHandler):
             inpt_file = urlparse(inpt.get('href')).path
             inpt_file = os.path.abspath(inpt_file)
             os.symlink(inpt_file, tmp_file)
-            LOGGER.debug("Linked input file %s to %s.", inpt_file, tmp_file)
+            LOGGER.debug("Linked input file {} to {}.".format(inpt_file, tmp_file))
         except Exception:
             # TODO: handle os.symlink on windows
-            # raise NoApplicableCode("Could not link file reference: %s" % e)
+            # raise NoApplicableCode("Could not link file reference: {}".format(e))
             LOGGER.warn("Could not link file reference")
             shutil.copy2(inpt_file, tmp_file)
 
@@ -960,8 +970,8 @@ class ComplexOutput(BasicIO, BasicComplex, IOHandler):
 
     def __init__(self, identifier, title=None, abstract=None, keywords=None,
                  workdir=None, data_format=None, supported_formats=None,
-                 mode=MODE.NONE):
-        BasicIO.__init__(self, identifier, title, abstract, keywords)
+                 mode=MODE.NONE, translations=None):
+        BasicIO.__init__(self, identifier, title, abstract, keywords, translations=translations)
         IOHandler.__init__(self, workdir=workdir, mode=mode)
         BasicComplex.__init__(self, data_format, supported_formats)
 
@@ -981,7 +991,9 @@ class ComplexOutput(BasicIO, BasicComplex, IOHandler):
     def get_url(self):
         """Return URL pointing to data
         """
-        (outtype, storage, url) = self.storage.store(self)
+        # TODO: it is not obvious that storing happens here
+        (_, _, url) = self.storage.store(self)
+        # url = self.storage.url(self)
         return url
 
 
