@@ -4,6 +4,7 @@
 ##################################################################
 
 import lxml.etree
+import requests
 from werkzeug.test import Client
 from werkzeug.wrappers import BaseResponse
 from pywps import __version__
@@ -19,6 +20,22 @@ import logging
 logging.disable(logging.CRITICAL)
 
 
+def service_ok(url, timeout=5):
+    try:
+        resp = requests.get(url, timeout=timeout)
+        if 'html' in resp.headers['content-type']:
+            ok = False
+        else:
+            ok = resp.ok
+    except requests.exceptions.ReadTimeout:
+        ok = False
+    except requests.exceptions.ConnectTimeout:
+        ok = False
+    except Exception:
+        ok = False
+    return ok
+
+
 class DocExampleProcess(Process):
     """This first line is going to be skipped by the :skiplines:1 option.
 
@@ -29,19 +46,26 @@ class DocExampleProcess(Process):
     """
 
     def __init__(self):
-        inputs = [LiteralInput('literal_input', "Literal input title", 'integer', "Literal input value abstract.",
-                               min_occurs=0, max_occurs=1, uoms=['meters', 'feet'], default=1),
-                  LiteralInput('date_input', 'The title is shown when no abstract is provided.', 'date',
-                               allowed_values=['2000-01-01', '2018-01-01']),
-                  ComplexInput('complex_input', 'Complex input title',
-                               [Format('application/json'), Format('application/x-netcdf')],
-                               abstract="Complex input abstract.", ),
-                  BoundingBoxInput('bb_input', 'BoundingBox input title', ['EPSG:4326', ],
-                                   metadata=[Metadata('EPSG.io', 'http://epsg.io/'), ]),
-                  ]
-        outputs = [LiteralOutput('literal_output', 'Literal output title', 'boolean', 'Boolean output abstract.',),
-                   ComplexOutput('complex_output', 'Complex output', [Format('text/plain'), ], ),
-                   BoundingBoxOutput('bb_output', 'BoundingBox output title', ['EPSG:4326', ])]
+        inputs = [
+            LiteralInput(
+                'literal_input', "Literal input title", 'integer', abstract="Literal input value abstract.",
+                min_occurs=0, max_occurs=1, uoms=['meters', 'feet'], default=1
+            ),
+            LiteralInput('date_input', 'The title is shown when no abstract is provided.', 'date',
+                         allowed_values=['2000-01-01', '2018-01-01']),
+            ComplexInput('complex_input', 'Complex input title',
+                         [Format('application/json'), Format('application/x-netcdf')],
+                         abstract="Complex input abstract.", ),
+            BoundingBoxInput('bb_input', 'BoundingBox input title', ['EPSG:4326', ],
+                             metadata=[Metadata('EPSG.io', 'http://epsg.io/'), ]),
+        ]
+        outputs = [
+            LiteralOutput(
+                'literal_output', 'Literal output title', 'boolean', abstract='Boolean output abstract.'
+            ),
+            ComplexOutput('complex_output', 'Complex output', [Format('text/plain'), ], ),
+            BoundingBoxOutput('bb_output', 'BoundingBox output title', ['EPSG:4326', ])
+        ]
 
         super(DocExampleProcess, self).__init__(
             self._handler,
@@ -49,7 +73,7 @@ class DocExampleProcess(Process):
             title="Process title",
             abstract="Multiline process abstract.",
             version="4.0",
-            metadata=[Metadata('PyWPS docs', 'http://pywps.org'),
+            metadata=[Metadata('PyWPS docs', 'https://pywps.org'),
                       Metadata('NumPy docstring conventions',
                                'https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt')],
             inputs=inputs,
