@@ -13,6 +13,7 @@ from pywps.app.basic import get_xpath_ns
 from pywps.app.Common import Metadata
 from pywps.inout.literaltypes import AllowedValue
 from pywps.validator.allowed_value import ALLOWEDVALUETYPE
+from pywps.configuration import CONFIG
 
 from pywps.tests import assert_pywps_version, client_for
 
@@ -28,6 +29,14 @@ def get_data_type(el):
     raise RuntimeError("Can't parse data type")
 
 
+def get_default_value(el):
+    default = None
+    xpath = xpath_ns(el, './DefaultValue')
+    if xpath:
+        default = xpath[0].text
+    return default
+
+
 xpath_ns = get_xpath_ns("1.0.0")
 
 
@@ -49,8 +58,9 @@ def get_describe_result(resp):
             if literal_data_el_list:
                 [literal_data_el] = literal_data_el_list
                 [data_type_el] = xpath_ns(literal_data_el, './ows:DataType')
+                default = get_default_value(literal_data_el)
                 data_type = get_data_type(data_type_el)
-                inputs.append((input_identifier, 'literal', data_type))
+                inputs.append((input_identifier, 'literal', data_type, default))
             elif complex_data_el_list:
                 [complex_data_el] = complex_data_el_list
                 formats = []
@@ -58,7 +68,7 @@ def get_describe_result(resp):
                                           './Supported/Format'):
                     [mimetype_el] = xpath_ns(format_el, './ows:MimeType')
                     formats.append({'mime_type': mimetype_el.text})
-                inputs.append((input_identifier, 'complex', formats))
+                inputs.append((input_identifier, 'complex', formats, None))
             else:
                 raise RuntimeError("Can't parse input description")
         result.append(ProcessDescription(identifier_el.text, inputs, metadata))
@@ -128,12 +138,94 @@ class DescribeProcessTest(unittest.TestCase):
         assert [pr.identifier for pr in result] == ['hello', 'ping']
 
 
+class DescribeProcessTranslationsTest(unittest.TestCase):
+
+    def setUp(self):
+        CONFIG.set('server', 'language', 'en-US,fr-CA')
+        self.client = client_for(
+            Service(
+                processes=[
+                    Process(
+                        lambda: None,
+                        "pr1",
+                        "Process 1",
+                        abstract="Process 1",
+                        inputs=[
+                            LiteralInput(
+                                'input1',
+                                title='Input name',
+                                abstract='Input abstract',
+                                translations={"fr-CA": {"title": "Nom de l'input", "abstract": "Description"}}
+                            )
+                        ],
+                        outputs=[
+                            LiteralOutput(
+                                'output1',
+                                title='Output name',
+                                abstract='Output abstract',
+                                translations={"fr-CA": {"title": "Nom de l'output", "abstract": "Description"}}
+                            )
+                        ],
+                        translations={"fr-CA": {"title": "Processus 1", "abstract": "Processus 1"}},
+                    ),
+                    Process(
+                        lambda: None,
+                        "pr2",
+                        "Process 2",
+                        abstract="Process 2",
+                        inputs=[
+                            LiteralInput(
+                                'input1',
+                                title='Input name',
+                                abstract='Input abstract',
+                                translations={"fr-CA": {"abstract": "Description"}}
+                            )
+                        ],
+                        outputs=[
+                            LiteralOutput(
+                                'output1',
+                                title='Output name',
+                                abstract='Output abstract',
+                                translations={"fr-CA": {"abstract": "Description"}}
+                            )
+                        ],
+                        translations={"fr-CA": {"title": "Processus 2"}},
+                    ),
+                ]
+            )
+        )
+
+    def tearDown(self):
+        CONFIG.set('server', 'language', 'en-US')
+
+    def test_get_describe_translations(self):
+        resp = self.client.get('?Request=DescribeProcess&service=wps&version=1.0.0&identifier=all&language=fr-CA')
+
+        assert resp.xpath('/wps:ProcessDescriptions/@xml:lang')[0] == "fr-CA"
+
+        titles = [e.text for e in resp.xpath('//ProcessDescription/ows:Title')]
+        abstracts = [e.text for e in resp.xpath('//ProcessDescription/ows:Abstract')]
+        assert titles == ['Processus 1', 'Processus 2']
+        assert abstracts == ['Processus 1', 'Process 2']
+
+        input_titles = [e.text for e in resp.xpath('//Input/ows:Title')]
+        input_abstracts = [e.text for e in resp.xpath('//Input/ows:Abstract')]
+
+        assert input_titles == ["Nom de l'input", "Input name"]
+        assert input_abstracts == ["Description", "Description"]
+
+        output_titles = [e.text for e in resp.xpath('//Output/ows:Title')]
+        output_abstracts = [e.text for e in resp.xpath('//Output/ows:Abstract')]
+
+        assert output_titles == ["Nom de l'output", "Output name"]
+        assert output_abstracts == ["Description", "Description"]
+
+
 class DescribeProcessInputTest(unittest.TestCase):
 
     def describe_process(self, process):
         client = client_for(Service(processes=[process]))
-        resp = client.get('?service=wps&version=1.0.0&Request=DescribeProcess&identifier=%s'
-                          % process.identifier)
+        resp = client.get('?service=wps&version=1.0.0&Request=DescribeProcess&identifier={}'.format(process.identifier))
         [result] = get_describe_result(resp)
         return result
 
@@ -150,7 +242,7 @@ class DescribeProcessInputTest(unittest.TestCase):
                 Metadata('process metadata 2', 'http://example.org/2')]
         )
         result = self.describe_process(hello_process)
-        assert result.inputs == [('the_name', 'literal', 'integer')]
+        assert result.inputs == [('the_name', 'literal', 'string', None)]
         assert result.metadata == ['process metadata 1', 'process metadata 2']
 
     def test_one_literal_integer_input(self):
@@ -162,7 +254,19 @@ class DescribeProcessInputTest(unittest.TestCase):
                                                      'Input number',
                                                      data_type='positiveInteger')])
         result = self.describe_process(hello_process)
-        assert result.inputs == [('the_number', 'literal', 'positiveInteger')]
+        assert result.inputs == [('the_number', 'literal', 'positiveInteger', None)]
+
+    def test_one_literal_integer_default_zero(self):
+        def hello(request):
+            pass
+        hello_process = Process(hello, 'hello',
+                                'Process Hello',
+                                inputs=[LiteralInput('the_number',
+                                                     'Input number',
+                                                     data_type='integer',
+                                                     default=0)])
+        result = self.describe_process(hello_process)
+        assert result.inputs == [('the_number', 'literal', 'integer', "0")]
 
 
 class InputDescriptionTest(unittest.TestCase):
@@ -249,10 +353,10 @@ class OutputDescriptionTest(unittest.TestCase):
         assert abstract.text == 'Description'
         assert keywords is not None
         assert len(kws) == 2
-        assert data_type.attrib['{%s}reference' % NAMESPACES['ows']] == OGCTYPE['string']
+        assert data_type.attrib['{{{}}}reference'.format(NAMESPACES['ows'])] == OGCTYPE['string']
         assert uoms is not None
         assert default_uom.text == 'metre'
-        assert default_uom.attrib['{%s}reference' % NAMESPACES['ows']] == OGCUNIT['metre']
+        assert default_uom.attrib['{{{}}}reference'.format(NAMESPACES['ows'])] == OGCUNIT['metre']
         assert len(supported_uoms) == 1
 
     def test_complex_output(self):
@@ -292,5 +396,6 @@ def load_tests(loader=None, tests=None, pattern=None):
         loader.loadTestsFromTestCase(DescribeProcessTest),
         loader.loadTestsFromTestCase(DescribeProcessInputTest),
         loader.loadTestsFromTestCase(InputDescriptionTest),
+        loader.loadTestsFromTestCase(DescribeProcessTranslationsTest),
     ]
     return unittest.TestSuite(suite_list)
