@@ -19,6 +19,7 @@ from pywps import E, get_ElementMakerForVersion
 from pywps.app.basic import get_xpath_ns
 from pywps._compat import text_type
 from pywps.tests import client_for, assert_response_success
+from pywps.configuration import CONFIG
 
 from pywps._compat import PY2
 from pywps._compat import StringIO
@@ -56,7 +57,7 @@ def create_greeter():
     def greeter(request, response):
         name = request.inputs['name'][0].data
         assert type(name) is text_type
-        response.outputs['message'].data = "Hello %s!" % name
+        response.outputs['message'].data = "Hello {}!".format(name)
         return response
 
     return Process(handler=greeter,
@@ -66,6 +67,39 @@ def create_greeter():
                    outputs=[LiteralOutput('message', 'Output message', data_type='string')])
 
 
+def create_translated_greeter():
+    def greeter(request, response):
+        name = request.inputs['name'][0].data
+        response.outputs['message'].data = "Hello {}!".format(name)
+        return response
+
+    return Process(
+        handler=greeter,
+        identifier='greeter',
+        title='Greeter',
+        abstract='Say hello',
+        inputs=[
+            LiteralInput(
+                'name',
+                'Input name',
+                data_type='string',
+                abstract='Input description',
+                translations={"fr-CA": {"title": "Nom", "abstract": "Description"}},
+            )
+        ],
+        outputs=[
+            LiteralOutput(
+                'message',
+                'Output message',
+                data_type='string',
+                abstract='Output description',
+                translations={"fr-CA": {"title": "Message de retour", "abstract": "Description"}},
+            )
+        ],
+        translations={"fr-CA": {"title": "Salutations", "abstract": "Dire allô"}},
+    )
+
+
 def create_bbox_process():
     def bbox_process(request, response):
         coords = request.inputs['mybbox'][0].data
@@ -164,13 +198,35 @@ def create_mimetype_process():
                    ])
 
 
+def create_metalink_process():
+    from .processes.metalinkprocess import MultipleOutputs
+    return MultipleOutputs()
+
+
 def get_output(doc):
+    """Return the content of LiteralData, Reference or ComplexData."""
+
     output = {}
     for output_el in xpath_ns(doc, '/wps:ExecuteResponse'
                                    '/wps:ProcessOutputs/wps:Output'):
         [identifier_el] = xpath_ns(output_el, './ows:Identifier')
-        [value_el] = xpath_ns(output_el, './wps:Data/wps:LiteralData')
-        output[identifier_el.text] = value_el.text
+
+        lit_el = xpath_ns(output_el, './wps:Data/wps:LiteralData')
+        if lit_el != []:
+            output[identifier_el.text] = lit_el[0].text
+
+        ref_el = xpath_ns(output_el, './wps:Reference')
+        if ref_el != []:
+            output[identifier_el.text] = ref_el[0].attrib['href']
+
+        data_el = xpath_ns(output_el, './wps:Data/wps:ComplexData')
+        if data_el != []:
+            if data_el[0].text:
+                output[identifier_el.text] = data_el[0].text
+            else:  # XML children
+                ch = list(data_el[0])[0]
+                output[identifier_el.text] = lxml.etree.tostring(ch)
+
     return output
 
 
@@ -178,6 +234,8 @@ class ExecuteTest(unittest.TestCase):
     """Test for Exeucte request KVP request"""
 
     def test_dods(self):
+        if PY2:
+            self.skipTest('fails on python 2.7')
         if not WITH_NC4:
             self.skipTest('netCDF4 not installed')
         my_process = create_complex_nc_process()
@@ -219,6 +277,7 @@ class ExecuteTest(unittest.TestCase):
             store_execute = False
             lineage = False
             outputs = ['conventions']
+            language = "en-US"
 
         request = FakeRequest()
 
@@ -249,6 +308,8 @@ class ExecuteTest(unittest.TestCase):
                 'mimeType': 'text/gml',
                 'data': 'the data'
             }]}
+            language = "en-US"
+
         request = FakeRequest()
 
         try:
@@ -304,6 +365,7 @@ class ExecuteTest(unittest.TestCase):
             outputs = {}
             store_execute = False
             lineage = False
+            language = "en-US"
 
         request = FakeRequest()
         response = service.execute('my_complex_process', request, 'fakeuuid')
@@ -333,6 +395,7 @@ class ExecuteTest(unittest.TestCase):
             raw = False
             store_execute = False
             lineage = False
+            language = "en-US"
 
         # valid mimetype
         request = FakeRequest('text/plain+test')
@@ -344,6 +407,11 @@ class ExecuteTest(unittest.TestCase):
         with self.assertRaises(InvalidParameterValue):
             response = service.execute('get_mimetype_process', request, 'fakeuuid')
 
+    def test_metalink(self):
+        client = client_for(Service(processes=[create_metalink_process()]))
+        resp = client.get('?Request=Execute&identifier=multiple-outputs')
+        assert resp.status_code == 400
+
     def test_missing_process_error(self):
         client = client_for(Service(processes=[create_ultimate_question()]))
         resp = client.get('?Request=Execute&identifier=foo')
@@ -384,7 +452,6 @@ class ExecuteTest(unittest.TestCase):
         assert get_output(resp.xml) == {'message': "Hello foo!"}
 
     def test_bbox(self):
-        self.skipTest('OWSlib not python 3 compatible')
         client = client_for(Service(processes=[create_bbox_process()]))
         request_doc = WPS.Execute(
             OWS.Identifier('my_bbox_process'),
@@ -408,13 +475,13 @@ class ExecuteTest(unittest.TestCase):
             output,
             './ows:Identifier')[0].text)
 
-        self.assertEqual('15 50', xpath_ns(
-            output,
-            './wps:Data/ows:BoundingBox/ows:LowerCorner')[0].text)
+        lower_corner = xpath_ns(output, './wps:Data/ows:WGS84BoundingBox/ows:LowerCorner')[0].text
+        lower_corner = lower_corner.strip().replace('  ', ' ')
+        self.assertEqual('15 50', lower_corner)
 
-        self.assertEqual('16 50', xpath_ns(
-            output,
-            './wps:Data/ows:BoundingBox/ows:LowerCorner')[0].text)
+        upper_corner = xpath_ns(output, './wps:Data/ows:WGS84BoundingBox/ows:UpperCorner')[0].text
+        upper_corner = upper_corner.strip().replace('  ', ' ')
+        self.assertEqual('16 51', upper_corner)
 
     def test_output_response_dataType(self):
         client = client_for(Service(processes=[create_greeter()]))
@@ -433,6 +500,58 @@ class ExecuteTest(unittest.TestCase):
         assert el.attrib['dataType'] == 'string'
 
 
+class ExecuteTranslationsTest(unittest.TestCase):
+
+    def setUp(self):
+        CONFIG.set('server', 'language', 'en-US,fr-CA')
+
+    def tearDown(self):
+        CONFIG.set('server', 'language', 'en-US')
+
+    def test_translations(self):
+        client = client_for(Service(processes=[create_translated_greeter()]))
+
+        request_doc = WPS.Execute(
+            OWS.Identifier('greeter'),
+            WPS.DataInputs(
+                WPS.Input(
+                    OWS.Identifier('name'),
+                    WPS.Data(WPS.LiteralData('foo'))
+                )
+            ),
+            WPS.ResponseForm(
+                WPS.ResponseDocument(
+                    lineage='true',
+                )
+            ),
+            version='1.0.0',
+            language='fr-CA',
+        )
+        resp = client.post_xml(doc=request_doc)
+
+        assert resp.xpath('/wps:ExecuteResponse/@xml:lang')[0] == "fr-CA"
+
+        process_title = [e.text for e in resp.xpath('//wps:Process/ows:Title')]
+        assert process_title == ["Salutations"]
+        process_abstract = [e.text for e in resp.xpath('//wps:Process/ows:Abstract')]
+        assert process_abstract == ["Dire allô"]
+
+        input_titles = [e.text for e in resp.xpath('//wps:Input/ows:Title')]
+        assert input_titles == ["Nom"]
+        input_abstract = [e.text for e in resp.xpath('//wps:Input/ows:Abstract')]
+        assert input_abstract == ["Description"]
+
+        output_titles = [e.text for e in resp.xpath('//wps:OutputDefinitions/wps:Output/ows:Title')]
+        assert output_titles == ["Message de retour"]
+        output_abstract = [e.text for e in resp.xpath('//wps:OutputDefinitions/wps:Output/ows:Abstract')]
+        assert output_abstract == ["Description"]
+
+        output_titles = [e.text for e in resp.xpath('//wps:ProcessOutputs/wps:Output/ows:Title')]
+        assert output_titles == ["Message de retour"]
+        output_abstract = [e.text for e in resp.xpath('//wps:ProcessOutputs/wps:Output/ows:Abstract')]
+        assert output_abstract == ["Description"]
+
+
 class ExecuteXmlParserTest(unittest.TestCase):
     """Tests for Execute request XML Parser
     """
@@ -613,6 +732,7 @@ def load_tests(loader=None, tests=None, pattern=None):
         loader = unittest.TestLoader()
     suite_list = [
         loader.loadTestsFromTestCase(ExecuteTest),
+        loader.loadTestsFromTestCase(ExecuteTranslationsTest),
         loader.loadTestsFromTestCase(ExecuteXmlParserTest),
     ]
     return unittest.TestSuite(suite_list)
