@@ -14,22 +14,25 @@ import tempfile
 import datetime
 import unittest
 import json
+from pywps import inout
 import base64
-from pywps import Format
+
+from pywps import Format, FORMATS
+from pywps.app.Common import Metadata
 from pywps.validator import get_validator
-from pywps import NAMESPACES
 from pywps.inout.basic import IOHandler, SOURCE_TYPE, SimpleHandler, BBoxInput, BBoxOutput, \
     ComplexInput, ComplexOutput, LiteralOutput, LiteralInput, _is_textfile
-from pywps.inout import BoundingBoxInput as BoundingBoxInputXML
-from pywps.inout.literaltypes import convert, AllowedValue
+from pywps.inout.literaltypes import convert, AllowedValue, AnyValue
+from pywps.inout.outputs import MetaFile, MetaLink, MetaLink4
 from pywps._compat import StringIO, text_type, urlparse
 from pywps.validator.base import emptyvalidator
 from pywps.exceptions import InvalidParameterValue
 from pywps.validator.mode import MODE
 from pywps.inout.basic import UOM
-from pywps.inout.storage import FileStorage
 from pywps._compat import PY2
-
+from pywps.inout.storage.file import FileStorageBuilder
+from pywps.tests import service_ok
+from pywps.translations import get_translation
 
 from lxml import etree
 
@@ -109,6 +112,8 @@ class IOHandlerTest(unittest.TestCase):
 
     def test_data(self):
         """Test data input IOHandler"""
+        if PY2:
+            self.skipTest('fails on python 2.7')
         self.iohandler.data = self._value
         self.iohandler.data_format = Format('foo', extension='.foo')
         self._test_outout(SOURCE_TYPE.DATA, '.foo')
@@ -128,9 +133,12 @@ class IOHandlerTest(unittest.TestCase):
         file_handler.close()
         self.iohandler.file = source
         self._test_outout(SOURCE_TYPE.FILE)
+        with self.assertRaises(TypeError):
+            self.iohandler[0].data = '5'
 
     def test_url(self):
-
+        if not service_ok('https://demo.mapserver.org'):
+            self.skipTest("mapserver is unreachable")
         wfsResource = 'http://demo.mapserver.org/cgi-bin/wfs?' \
                       'service=WFS&version=1.1.0&' \
                       'request=GetFeature&' \
@@ -155,6 +163,8 @@ class IOHandlerTest(unittest.TestCase):
         self.skipTest('Memory object not implemented')
 
     def test_data_bytes(self):
+        if PY2:
+            self.skipTest('fails on python 2.7')
         self._value = b'aa'
 
         self.iohandler.data = self._value
@@ -189,12 +199,12 @@ class ComplexInputTest(unittest.TestCase):
     def setUp(self):
         self.tmp_dir = tempfile.mkdtemp()
         data_format = get_data_format('application/json')
-        self.complex_in = ComplexInput(identifier="complexinput",
-                                       title='MyComplex',
-                                       abstract='My complex input',
-                                       keywords=['kw1', 'kw2'],
-                                       workdir=self.tmp_dir,
-                                       supported_formats=[data_format])
+        self.complex_in = inout.inputs.ComplexInput(identifier="complexinput",
+                                                    title='MyComplex',
+                                                    abstract='My complex input',
+                                                    keywords=['kw1', 'kw2'],
+                                                    workdir=self.tmp_dir,
+                                                    supported_formats=[data_format])
 
         self.complex_in.data = "Hallo world!"
 
@@ -217,21 +227,198 @@ class ComplexInputTest(unittest.TestCase):
     def test_data_format(self):
         self.assertIsInstance(self.complex_in.supported_formats[0], Format)
 
-    def test_json_out(self):
-        self.skipTest('json property now in pywps.inout.inputs.ComplexInput')
-        out = self.complex_in.json
 
-        self.assertEqual(out['workdir'], self.tmp_dir, 'Workdir defined')
-        self.assertTrue(out['file'], 'There is no file')
-        self.assertTrue(out['supported_formats'], 'There are some formats')
-        self.assertEqual(len(out['supported_formats']), 1, 'There is one formats')
-        self.assertEqual(out['title'], 'MyComplex', 'Title not set but existing')
-        self.assertEqual(out['abstract'], 'My complex input', 'Abstract not set but existing')
-        self.assertEqual(out['keywords'], ['kw1', 'kw2'], 'Keywords not set but existing')
-        self.assertEqual(out['identifier'], 'complexinput', 'identifier set')
-        self.assertEqual(out['type'], 'complex', 'it is complex input')
-        self.assertTrue(out['data_format'], 'data_format set')
-        self.assertEqual(out['data_format']['mime_type'], 'application/json', 'data_format set')
+class SerializationComplexInputTest(unittest.TestCase):
+    """ComplexInput test cases"""
+
+    def setUp(self):
+        self.tmp_dir = tempfile.mkdtemp()
+
+    def make_complex_input(self):
+        complex = inout.inputs.ComplexInput(
+            identifier="complexinput",
+            title='MyComplex',
+            abstract='My complex input',
+            keywords=['kw1', 'kw2'],
+            workdir=self.tmp_dir,
+            supported_formats=[get_data_format('application/json')],
+            metadata=[Metadata("special data")],
+            default="/some/file/path",
+            default_type=SOURCE_TYPE.FILE,
+            translations={"fr-CA": {"title": "Mon input", "abstract": "Une description"}},
+        )
+        complex.as_reference = False
+        complex.method = "GET"
+        complex.max_size = 1000
+        return complex
+
+    def assert_complex_equals(self, complex_1, complex_2):
+        self.assertEqual(complex_1.identifier, complex_2.identifier)
+        self.assertEqual(complex_1.title, complex_2.title)
+        self.assertEqual(complex_1.supported_formats, complex_2.supported_formats)
+        self.assertEqual(complex_1.data_format, complex_2.data_format)
+        self.assertEqual(complex_1.abstract, complex_2.abstract)
+        self.assertEqual(complex_1.keywords, complex_2.keywords)
+        self.assertEqual(complex_1.workdir, complex_2.workdir)
+        self.assertEqual(complex_1.metadata, complex_2.metadata)
+        self.assertEqual(complex_1.max_occurs, complex_2.max_occurs)
+        self.assertEqual(complex_1.valid_mode, complex_2.valid_mode)
+        self.assertEqual(complex_1.as_reference, complex_2.as_reference)
+        self.assertEqual(complex_1.translations, complex_2.translations)
+
+        self.assertEqual(complex_1.prop, complex_2.prop)
+
+        if complex_1.prop != 'url':
+            # don't download the file when running tests
+            self.assertEqual(complex_1.file, complex_2.file)
+            self.assertEqual(complex_1.data, complex_2.data)
+
+        self.assertEqual(complex_1.url, complex_2.url)
+
+    def test_complex_input_file(self):
+        complex = self.make_complex_input()
+        some_file = os.path.join(self.tmp_dir, "some_file.txt")
+        with open(some_file, "w") as f:
+            f.write("some data")
+        complex.file = some_file
+        complex2 = inout.inputs.ComplexInput.from_json(complex.json)
+        self.assert_complex_equals(complex, complex2)
+        self.assertEqual(complex.prop, 'file')
+
+    def test_complex_input_data(self):
+        complex = self.make_complex_input()
+        complex.data = "some data"
+        complex2 = inout.inputs.ComplexInput.from_json(complex.json)
+        # the data is enclosed by a CDATA tag
+        complex._data = u'<![CDATA[{}]]>'.format(complex.data)
+        # it's expected that the file path changed
+        complex._file = complex2.file
+
+        self.assert_complex_equals(complex, complex2)
+        self.assertEqual(complex.prop, 'data')
+
+    def test_complex_input_stream(self):
+        complex = self.make_complex_input()
+        complex.stream = StringIO("some data")
+        complex2 = inout.inputs.ComplexInput.from_json(complex.json)
+
+        # the serialized stream becomes a data type
+        # we hard-code it for the testing comparison
+        complex.prop = 'data'
+        # the data is enclosed by a CDATA tag
+        complex._data = u'<![CDATA[{}]]>'.format(complex.data)
+        # it's expected that the file path changed
+        complex._file = complex2.file
+
+        self.assert_complex_equals(complex, complex2)
+
+    def test_complex_input_url(self):
+        complex = self.make_complex_input()
+        complex.url = "http://test.opendap.org:80/opendap/netcdf/examples/sresa1b_ncar_ccsm3_0_run1_200001.nc"
+        complex2 = inout.inputs.ComplexInput.from_json(complex.json)
+        self.assert_complex_equals(complex, complex2)
+        self.assertEqual(complex.prop, 'url')
+
+
+class SerializationLiteralInputTest(unittest.TestCase):
+    """LiteralInput test cases"""
+
+    def setUp(self):
+        self.tmp_dir = tempfile.mkdtemp()
+
+    def make_literal_input(self):
+        literal = inout.inputs.LiteralInput(
+            identifier="complexinput",
+            title='MyComplex',
+            data_type='string',
+            workdir=self.tmp_dir,
+            abstract="some description",
+            keywords=['kw1', 'kw2'],
+            metadata=[Metadata("special data")],
+            uoms=['metre', 'unity'],
+            min_occurs=2,
+            max_occurs=5,
+            mode=MODE.STRICT,
+            allowed_values=[AllowedValue(value='something'), AllowedValue(value='something else'), AnyValue()],
+            default="something else",
+            default_type=SOURCE_TYPE.DATA,
+        )
+        literal.data = 'something'
+        literal.uom = UOM('unity')
+        literal.as_reference = False
+        return literal
+
+    def assert_literal_equals(self, literal_1, literal_2):
+        self.assertEqual(literal_1.identifier, literal_2.identifier)
+        self.assertEqual(literal_1.title, literal_2.title)
+        self.assertEqual(literal_1.data_type, literal_2.data_type)
+        self.assertEqual(literal_1.workdir, literal_2.workdir)
+        self.assertEqual(literal_1.abstract, literal_2.abstract)
+        self.assertEqual(literal_1.keywords, literal_2.keywords)
+        self.assertEqual(literal_1.metadata, literal_2.metadata)
+        self.assertEqual(literal_1.uoms, literal_2.uoms)
+        self.assertEqual(literal_1.min_occurs, literal_2.min_occurs)
+        self.assertEqual(literal_1.max_occurs, literal_2.max_occurs)
+        self.assertEqual(literal_1.valid_mode, literal_2.valid_mode)
+        self.assertEqual(literal_1.allowed_values, literal_2.allowed_values)
+        self.assertEqual(literal_1.any_value, literal_2.any_value)
+        self.assertTrue(literal_1.any_value)
+        self.assertEqual(literal_1.as_reference, literal_2.as_reference)
+
+        self.assertEqual(literal_1.data, literal_2.data)
+
+    def test_literal_input(self):
+        literal = self.make_literal_input()
+        literal2 = inout.inputs.LiteralInput.from_json(literal.json)
+        self.assert_literal_equals(literal, literal2)
+
+
+class SerializationBoundingBoxInputTest(unittest.TestCase):
+    """LiteralInput test cases"""
+
+    def setUp(self):
+        self.tmp_dir = tempfile.mkdtemp()
+
+    def make_bbox_input(self):
+        bbox = inout.inputs.BoundingBoxInput(
+            identifier="complexinput",
+            title='MyComplex',
+            crss=['epsg:3857', 'epsg:4326'],
+            abstract="some description",
+            keywords=['kw1', 'kw2'],
+            dimensions=2,
+            workdir=self.tmp_dir,
+            metadata=[Metadata("special data")],
+            min_occurs=2,
+            max_occurs=5,
+            mode=MODE.NONE,
+            default="something else",
+            default_type=SOURCE_TYPE.DATA,
+        )
+        bbox.as_reference = False
+        return bbox
+
+    def assert_bbox_equals(self, bbox_1, bbox_2):
+        self.assertEqual(bbox_1.identifier, bbox_2.identifier)
+        self.assertEqual(bbox_1.title, bbox_2.title)
+        self.assertEqual(bbox_1.crss, bbox_2.crss)
+        self.assertEqual(bbox_1.abstract, bbox_2.abstract)
+        self.assertEqual(bbox_1.keywords, bbox_2.keywords)
+        self.assertEqual(bbox_1.dimensions, bbox_2.dimensions)
+        self.assertEqual(bbox_1.workdir, bbox_2.workdir)
+        self.assertEqual(bbox_1.metadata, bbox_2.metadata)
+        self.assertEqual(bbox_1.min_occurs, bbox_2.min_occurs)
+        self.assertEqual(bbox_1.max_occurs, bbox_2.max_occurs)
+        self.assertEqual(bbox_1.valid_mode, bbox_2.valid_mode)
+        self.assertEqual(bbox_1.as_reference, bbox_2.as_reference)
+
+        self.assertEqual(bbox_1.ll, bbox_2.ll)
+        self.assertEqual(bbox_1.ur, bbox_2.ur)
+
+    def test_bbox_input(self):
+        bbox = self.make_bbox_input()
+        bbox2 = inout.inputs.BoundingBoxInput.from_json(bbox.json)
+        self.assert_bbox_equals(bbox, bbox2)
 
 
 class DodsComplexInputTest(unittest.TestCase):
@@ -252,7 +439,7 @@ class DodsComplexInputTest(unittest.TestCase):
 
     def test_validator(self):
         self.assertEqual(self.complex_in.data_format.validate,
-                       get_validator('application/x-ogc-dods'))
+                         get_validator('application/x-ogc-dods'))
         self.assertEqual(self.complex_in.validator,
                          get_validator('application/x-ogc-dods'))
         frmt = get_data_format('application/x-ogc-dods')
@@ -273,15 +460,23 @@ class ComplexOutputTest(unittest.TestCase):
     def setUp(self):
         tmp_dir = tempfile.mkdtemp()
         data_format = get_data_format('application/json')
-        self.complex_out = ComplexOutput(identifier="complexinput", workdir=tmp_dir,
-                                         data_format=data_format,
-                                         supported_formats=[data_format],
-                                         mode=MODE.NONE)
+        self.complex_out = inout.outputs.ComplexOutput(
+            identifier="complexoutput",
+            title='Complex Output',
+            workdir=tmp_dir,
+            data_format=data_format,
+            supported_formats=[data_format],
+            mode=MODE.NONE,
+            translations={"fr-CA": {"title": "Mon output", "abstract": "Une description"}},
+            )
 
-        self.complex_out_nc = ComplexOutput(identifier="netcdf", workdir=tmp_dir,
-                                            data_format=get_data_format('application/x-netcdf'),
-                                            supported_formats=[get_data_format('application/x-netcdf')],
-                                            mode=MODE.NONE)
+        self.complex_out_nc = inout.outputs.ComplexOutput(
+            identifier="netcdf",
+            title="NetCDF output",
+            workdir=tmp_dir,
+            data_format=get_data_format('application/x-netcdf'),
+            supported_formats=[get_data_format('application/x-netcdf')],
+            mode=MODE.NONE)
 
         self.data = json.dumps({'a': 1, 'unicodé': u'éîïç', })
         self.ncfile = os.path.join(DATA_DIR, 'netcdf', 'time.nc')
@@ -323,10 +518,14 @@ class ComplexOutputTest(unittest.TestCase):
             self.assertEqual(f.read(), self.data)
 
     def test_file_handler_netcdf(self):
+        if PY2:
+            self.skipTest('fails on python 2.7')
         self.complex_out_nc.file = self.ncfile
-        data = self.complex_out_nc.base64
+        self.complex_out_nc.base64
 
     def test_data_handler(self):
+        if PY2:
+            self.skipTest('fails on python 2.7')
         self.complex_out.data = self.data
         with open(self.complex_out.file) as f:
             self.assertEqual(f.read(), self.data)
@@ -345,11 +544,19 @@ class ComplexOutputTest(unittest.TestCase):
                       'request=GetFeature&' \
                       'typename=continents&maxfeatures=2'
         self.complex_out.url = wfsResource
-        storage = FileStorage()
-        self.complex_out.storage = storage
+        self.complex_out.storage = FileStorageBuilder().build()
         url = self.complex_out.get_url()
         self.assertEqual('file', urlparse(url).scheme)
 
+    def test_json(self):
+        new_output = inout.outputs.ComplexOutput.from_json(self.complex_out.json)
+        self.assertEqual(new_output.identifier, 'complexoutput')
+        self.assertEqual(
+            new_output.translations,
+            {"fr-ca": {"title": "Mon output", "abstract": "Une description"}},
+            'translations does not exist'
+        )
+
 
 class SimpleHandlerTest(unittest.TestCase):
     """SimpleHandler test cases"""
@@ -372,12 +579,16 @@ class LiteralInputTest(unittest.TestCase):
 
     def setUp(self):
 
-        self.literal_input = LiteralInput(
+        self.literal_input = inout.inputs.LiteralInput(
             identifier="literalinput",
+            title="Literal Input",
+            data_type='integer',
             mode=2,
             allowed_values=(1, 2, (3, 3, 12)),
             default=6,
-            uoms=(UOM("metre"),))
+            uoms=(UOM("metre"),),
+            translations={"fr-CA": {"title": "Mon input", "abstract": "Une description"}},
+        )
 
     def test_contruct(self):
         self.assertIsInstance(self.literal_input, LiteralInput)
@@ -410,53 +621,73 @@ class LiteralInputTest(unittest.TestCase):
         out = self.literal_input.json
 
         self.assertTrue('uoms' in out, 'UOMs does not exist')
-        self.assertTrue('uom' in out, 'uom exists')
+        self.assertTrue('uom' in out, 'uom does not exist')
         self.assertFalse(out['workdir'], 'Workdir exist')
         self.assertEqual(out['data_type'], 'integer', 'Data type is integer')
         self.assertFalse(out['abstract'], 'abstract exist')
         self.assertFalse(out['keywords'], 'keywords exist')
-        self.assertFalse(out['title'], 'title exist')
-        self.assertEqual(out['data'], 9, 'data set')
+        self.assertTrue(out['title'], 'title does not exist')
+        self.assertEqual(out['data'], '9', 'data set')
         self.assertEqual(out['mode'], MODE.STRICT, 'Mode set')
         self.assertEqual(out['identifier'], 'literalinput', 'identifier set')
         self.assertEqual(out['type'], 'literal', 'it\'s literal input')
         self.assertEqual(len(out['allowed_values']), 3, '3 allowed values')
         self.assertEqual(out['allowed_values'][0]['value'], 1, 'allowed value 1')
+        self.assertEqual(
+            out['translations'],
+            {"fr-ca": {"title": "Mon input", "abstract": "Une description"}},
+            'translations does not exist'
+        )
 
     def test_json_out_datetime(self):
-        inpt = LiteralInput(
+        inpt = inout.inputs.LiteralInput(
             identifier="datetime",
+            title="Literal Input",
             mode=2,
             data_type='dateTime')
         inpt.data = "2017-04-20T12:30:00"
         out = inpt.json
-        self.assertEqual(out['data'], datetime.datetime(2017, 4, 20, 12, 30, 0), 'datetime set')
+        self.assertEqual(out['data'], '2017-04-20 12:30:00', 'datetime set')
 
     def test_json_out_time(self):
-        inpt = LiteralInput(
+        inpt = inout.inputs.LiteralInput(
             identifier="time",
+            title="Literal Input",
             mode=2,
             data_type='time')
         inpt.data = "12:30:00"
         out = inpt.json
-        self.assertEqual(out['data'], datetime.time(12, 30, 0), 'time set')
+        self.assertEqual(out['data'], '12:30:00', 'time set')
 
     def test_json_out_date(self):
-        inpt = LiteralInput(
+        inpt = inout.inputs.LiteralInput(
             identifier="date",
+            title="Literal Input",
             mode=2,
             data_type='date')
         inpt.data = "2017-04-20"
         out = inpt.json
-        self.assertEqual(out['data'], datetime.date(2017, 4, 20), 'date set')
+        self.assertEqual(out['data'], '2017-04-20', 'date set')
 
+    def test_translations(self):
+        title_fr = get_translation(self.literal_input, "title", "fr-CA")
+        assert title_fr == "Mon input"
+        abstract_fr = get_translation(self.literal_input, "abstract", "fr-CA")
+        assert abstract_fr == "Une description"
+        identifier = get_translation(self.literal_input, "identifier", "fr-CA")
+        assert identifier == self.literal_input.identifier
 
 class LiteralOutputTest(unittest.TestCase):
     """LiteralOutput test cases"""
 
     def setUp(self):
 
-        self.literal_output = LiteralOutput("literaloutput", data_type="integer")
+        self.literal_output = inout.outputs.LiteralOutput(
+            "literaloutput", 
+            data_type="integer", 
+            title="Literal Output",
+            translations={"fr-CA": {"title": "Mon output", "abstract": "Une description"}},
+        )
 
     def test_contruct(self):
         self.assertIsInstance(self.literal_output, LiteralOutput)
@@ -468,15 +699,28 @@ class LiteralOutputTest(unittest.TestCase):
         self.literal_output.store = storage
         self.assertEqual(self.literal_output.store, storage)
 
+    def test_json(self):
+        new_output = inout.outputs.LiteralOutput.from_json(self.literal_output.json)
+        self.assertEqual(new_output.identifier, 'literaloutput')
+        self.assertEqual(
+            new_output.translations,
+            {"fr-ca": {"title": "Mon output", "abstract": "Une description"}},
+            'translations does not exist'
+        )
 
-class BoxInputTest(unittest.TestCase):
-    """BBoxInput test cases"""
+
+class BBoxInputTest(unittest.TestCase):
+    """BountingBoxInput test cases"""
 
     def setUp(self):
 
-        self.bbox_input = BBoxInput("bboxinput", dimensions=2)
-        self.bbox_input.ll = [0, 1]
-        self.bbox_input.ur = [2, 4]
+        self.bbox_input = inout.inputs.BoundingBoxInput(
+            "bboxinput", 
+            title="BBox input", 
+            dimensions=2,
+            translations={"fr-CA": {"title": "Mon input", "abstract": "Une description"}},
+        )
+        self.bbox_input.data = [0, 1, 2, 4]
 
     def test_contruct(self):
         self.assertIsInstance(self.bbox_input, BBoxInput)
@@ -485,19 +729,30 @@ class BoxInputTest(unittest.TestCase):
         out = self.bbox_input.json
 
         self.assertTrue(out['identifier'], 'identifier exists')
-        self.assertFalse(out['title'], 'title exists')
+        self.assertTrue(out['title'], 'title does not exist')
         self.assertFalse(out['abstract'], 'abstract set')
         self.assertEqual(out['type'], 'bbox', 'type set')
-        self.assertTupleEqual(out['bbox'], ([0, 1], [2, 4]), 'data are tehre')
+        # self.assertTupleEqual(out['bbox'], ([0, 1], [2, 4]), 'data are there')
+        self.assertEqual(out['bbox'], [0, 1, 2, 4], 'data are there')
         self.assertEqual(out['dimensions'], 2, 'Dimensions set')
+        self.assertEqual(
+            out['translations'],
+            {"fr-ca": {"title": "Mon input", "abstract": "Une description"}},
+            'translations does not exist'
+        )
 
 
-class BoxOutputTest(unittest.TestCase):
+class BBoxOutputTest(unittest.TestCase):
     """BoundingBoxOutput test cases"""
 
     def setUp(self):
-
-        self.bbox_out = BBoxOutput("bboxoutput")
+        self.bbox_out = inout.outputs.BoundingBoxOutput(
+            "bboxoutput",
+            title="BBox output",
+            dimensions=2,
+            crss=['epsg:3857', 'epsg:4326'],
+            translations={"fr-CA": {"title": "Mon output", "abstract": "Une description"}},
+        )
 
     def test_contruct(self):
         self.assertIsInstance(self.bbox_out, BBoxOutput)
@@ -509,6 +764,74 @@ class BoxOutputTest(unittest.TestCase):
         self.bbox_out.store = storage
         self.assertEqual(self.bbox_out.store, storage)
 
+    def test_json(self):
+        new_bbox = inout.outputs.BoundingBoxOutput.from_json(self.bbox_out.json)
+        self.assertEqual(new_bbox.identifier, 'bboxoutput')
+        self.assertEqual(
+            new_bbox.translations,
+            {"fr-ca": {"title": "Mon output", "abstract": "Une description"}},
+            'translations does not exist'
+        )
+
+
+class TestMetaLink(unittest.TestCase):
+    tmp_dir = tempfile.mkdtemp()
+
+    def metafile(self):
+        mf = MetaFile('identifier', 'title', fmt=FORMATS.JSON)
+        mf.data = json.dumps({'a': 1})
+        mf._set_workdir(self.tmp_dir)
+        return mf
+
+    def test_metafile(self):
+        mf = self.metafile()
+        self.assertEqual('identifier', mf.identity)
+
+    def metalink(self):
+        ml = MetaLink(identity='unittest', description='desc', files=(self.metafile(), ), workdir=self.tmp_dir)
+        return ml
+
+    def metalink4(self):
+        ml = MetaLink4(identity='unittest', description='desc', files=(self.metafile(), ), workdir=self.tmp_dir)
+        return ml
+
+    def test_metalink(self):
+        from pywps.validator.complexvalidator import validatexml
+
+        out = inout.outputs.ComplexOutput('metatest', 'MetaLink Test title', abstract='MetaLink test abstract',
+                                          supported_formats=[FORMATS.METALINK, ],
+                                          as_reference=True)
+        out.workdir = self.tmp_dir
+        ml = self.metalink()
+
+        out.data = ml.xml
+        self.assertTrue(validatexml(out, MODE.STRICT))
+
+    def test_metalink4(self):
+        from pywps.validator.complexvalidator import validatexml
+
+        out = inout.outputs.ComplexOutput('metatest', 'MetaLink4 Test title', abstract='MetaLink4 test abstract',
+                                          supported_formats=[FORMATS.META4, ],
+                                          as_reference=True)
+        out.workdir = self.tmp_dir
+        ml = self.metalink4()
+
+        out.data = ml.xml
+        self.assertTrue(validatexml(out, MODE.STRICT))
+
+    def test_hash(self):
+        ml = self.metalink()
+        assert 'hash' not in ml.xml
+
+        ml.checksums = True
+        assert 'hash' in ml.xml
+
+        ml4 = self.metalink4()
+        assert 'hash' not in ml4.xml
+
+        ml4.checksums = True
+        assert 'hash' in ml4.xml
+
 
 def load_tests(loader=None, tests=None, pattern=None):
     if not loader:
@@ -518,10 +841,13 @@ def load_tests(loader=None, tests=None, pattern=None):
         loader.loadTestsFromTestCase(ComplexInputTest),
         loader.loadTestsFromTestCase(DodsComplexInputTest),
         loader.loadTestsFromTestCase(ComplexOutputTest),
+        loader.loadTestsFromTestCase(SerializationBoundingBoxInputTest),
+        loader.loadTestsFromTestCase(SerializationComplexInputTest),
+        loader.loadTestsFromTestCase(SerializationLiteralInputTest),
         loader.loadTestsFromTestCase(SimpleHandlerTest),
         loader.loadTestsFromTestCase(LiteralInputTest),
         loader.loadTestsFromTestCase(LiteralOutputTest),
-        loader.loadTestsFromTestCase(BoxInputTest),
-        loader.loadTestsFromTestCase(BoxOutputTest)
+        loader.loadTestsFromTestCase(BBoxInputTest),
+        loader.loadTestsFromTestCase(BBoxOutputTest)
     ]
     return unittest.TestSuite(suite_list)
